/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
 *                                                                         *
 ***************************************************************************/


#include "KCDemuDeviceMenu.h"
#include "KCDemuDeviceMenu.moc"

//Create new device menu (for given device number)
KCDemuDeviceMenu::KCDemuDeviceMenu(QWidget* parent, const char* name, int devNum) {
	//Set device number
	this->deviceNum = devNum;

	//Check if module file is there, if not module is not loaded
	QFile *netInfo = new QFile( "/proc/cdemu" );	//Info about kernel module
	if( ! netInfo->open( IO_ReadOnly ) ) {
		KMessageBox::error( this, "cdemu module is not loaded (and I couldn't load it myself)!\nPlease load it before running KCDemu!", "Module error" );
		exit( 0 );
	}
	//Close file and delete handler
	netInfo->close();
	delete netInfo;
}

KCDemuDeviceMenu::~KCDemuDeviceMenu() {
}

//Reads status from /proc/cdemu
void KCDemuDeviceMenu::checkStatus() {
	bool bFileMounted = false;			//Is a file mounted?
	QString data;					//Data buffer for file content

	//Check if module file is there, if not module is not loaded
	QFile *netInfo = new QFile( "/proc/cdemu" );	//Info about kernel module
	if( ! netInfo->open( IO_ReadOnly ) ) {
		KMessageBox::error( this, "cdemu module is not loaded (and I couldn't load it myself)!\nPlease load it before running KCDemu!", "Module error" );
		exit( 0 );
	}

	//Read lines till we got the line for our device (+1 since the first line is a header)
	for( int i = 0; i <= (this->deviceNum + 1); i++ ) {
		netInfo->readLine( data, 4096 );
	}
	//Close file and delete handler
	netInfo->close();
	delete netInfo;

	//Check our line for info on the mounted image
	QRegExp devInfo( "\\s+(\\d+):\\s+(\\d+)\\s+(.+)" );
	devInfo.search( data );
	QString device = devInfo.cap( 1 );	//device number
	QString loaded = devInfo.cap( 2 );	//0 = no image loaded, 1 = image loaded
	QString image = devInfo.cap( 3 );	//image name (if loaded)

	QString fileName( "No file mounted" );
	//If loaded, set properties
	if( loaded.compare( "1" ) == 0 ) {
		bFileMounted = true;
		fileName = image;
		fileName.prepend( "Mounted: " );
		this->currentCDImage = image;
		emit CDImageOpened( this->deviceNum, true );
	}

	//Create menu
	//this->setAsDefaultID = insertItem( "Set as /dev/cdrom", this, SLOT( setAsCdrom() ) );
	this->unmountEntryID = insertItem( "Unmount", this, SLOT( unmountCDImage() ) );
	setItemEnabled( this->unmountEntryID, bFileMounted);
	insertSeparator();
	this->fileNameEntryID = insertItem( fileName, this, SLOT( openCDImage() ) );
}

//Open an image file
void KCDemuDeviceMenu::openCDImage() {
	//Get image file name
	QString fileName = KFileDialog::getOpenFileName( QString::null, "*.cue|cue sheet files\n*.bin|bin image files", this, "Open image file" );

	//Check if user really has chosen a file
	if( fileName.isEmpty() ) {
		printf( "No file selected!\n" );
		return;
	}

	//If a image is mounted atm, unmount it
	if( !this->currentCDImage.isEmpty() ) {
		this->unmountCDImage();
	}

//	printf( "Chosen file is: %s\n", fileName.ascii() );

	printf( "Selected file: %s\n", fileName.ascii() );

	//Kreate process and run cdemu command
	KCDemuProcess *mountCommand = new KCDemuProcess();
	*mountCommand << "cdemu";
	*mountCommand << QString::number( this->deviceNum ) << fileName;
	mountCommand->start( KCDemuProcess::Block, KProcess::AllOutput );

	if( mountCommand->gotError() == false ) {
		//Save current mounted image name
		this->currentCDImage = fileName;
	
		fileName = fileName.prepend( "Mounted: " );
		changeItem( this->fileNameEntryID, fileName );
		setItemEnabled( this->unmountEntryID, true);
	}
	delete mountCommand;

	emit CDImageOpened( this->deviceNum, true );
}

//Unmount an CD-Image
void KCDemuDeviceMenu::unmountCDImage() {
	//Check if an image is mounted atm
	if( this->currentCDImage.isEmpty() ) {
		return;
	}

	KCDemuProcess *unmountCommand = new KCDemuProcess();
	*unmountCommand << "cdemu";
	*unmountCommand << "-u" << QString::number( this->deviceNum );
	unmountCommand->start( KCDemuProcess::Block, KProcess::AllOutput );
	delete unmountCommand;

	//Reset current mounted image name
	this->currentCDImage = QString::null;

	//Set labels / enable states
	changeItem( this->fileNameEntryID, "No file mounted" );
	setItemEnabled( this->unmountEntryID, false);

	emit CDImageOpened( this->deviceNum, false);
}

//Set as "/dev/cdrom"
void KCDemuDeviceMenu::setAsCdrom() {
	QFileInfo CDInfo( "/dev/cdrom" );

	if( ! CDInfo.isSymLink() ) {
		KMessageBox::error( this, "/dev/cdrom is no link, wont touch any real devices", "Error" );
		return;
	}

	QString realFile = CDInfo.dirPath();	//get path
	realFile.append( "/" );			//Complete path
	realFile.append( CDInfo.readLink() );	//append file name (real name)

	if( realFile.isEmpty() ) {
		KMessageBox::error( this, "/dev/cdrom points nowhere, that's really strange", "Error" );
		return;
	}

	//If /dev/cdrom hasn't been changed before, save old position
	if( realFile.find( "/dev/cdemu" ) == -1 ) {
		KCDemuProcess *saveCommand = new KCDemuProcess();
		*saveCommand << "ln";
		*saveCommand << "-s" << realFile << "/dev/cdrom_old" ;
		saveCommand->start( KCDemuProcess::Block, KProcess::AllOutput );
		delete saveCommand;

		KMessageBox::information( this, "Saved original position", "Information" );
	}

	//Now set link to this device
	QString device( "/dev/cdemu/" );
	device.append( QString::number( this->deviceNum ) );

	KCDemuProcess *linkCommand = new KCDemuProcess();
	*linkCommand << "ln";
	*linkCommand << "-f" << "-s" << device << "/dev/cdrom" ;
	linkCommand->start( KCDemuProcess::Block, KProcess::AllOutput );
	delete linkCommand;

}
